#include "GUIFilter.h"

int LE; // LE650 device handler
		
bool MyApp::OnInit()
{
	// command option //
	wxCmdLineEntryDesc cmdLineDesc[] = {
		{wxCMD_LINE_PARAM, // parameter
				NULL, NULL,
		 ("Serial#"), // help text
		 wxCMD_LINE_VAL_STRING, // string input
		 wxCMD_LINE_PARAM_OPTIONAL // optional
		},
		{wxCMD_LINE_NONE} // end of the list
	};

	wxCmdLineParser parser(cmdLineDesc, argc, argv);
	parser.Parse();
	
	LE = -1;

	// Init //
	if( LEU_Init() != 0){
		std::cout << "Init Error\n" << std::endl;
		GetErrStatus();
		return false;
	}

	// LE connection //
	if(argc<2){
		// auto connect w/o serial
		std::cout << "auto connection without serial#" << std::endl;
		LE = LEU_ConnectNoSerial(1024*1024);
	}
	else{
		char * serial = new char[argv[1].length() + 1];
		strcpy(serial, argv[1].c_str());
		std::cout << "connection serial# = " << serial << std::endl;
		LE = LEU_Connect(serial, 1024*1024);
		delete[] serial;
	}
	
	if(LE == LEU_FAIL) {
		std::cout << "failed\n" << std::endl;
		GetErrStatus();
		LEU_End();
		return false;
	}

	MyFrame *frame = new MyFrame("LE650H2 GUI Sample");
	
    frame->Show(true);

    return true;
}

MyFrame::MyFrame(const wxString& title)
  : wxFrame(NULL, wxID_ANY, title, wxPoint(50,50), wxSize(MYWIN_WID,MYWIN_HGT)),
    timer(this, ID_TIMER)
{
	// menubar //
    wxMenu *fileMenu = new wxMenu;
	fileMenu->Append(Menu_Quit, "E&xit\tAlt-X", "Quit this program");

    wxMenuBar *menuBar = new wxMenuBar();
    menuBar->Append(fileMenu, "&File");
	
    SetMenuBar(menuBar);

	int ww = MYWIN_WID;
	int hh = 100;
	// control region //
	btnPanel = new wxPanel(this, wxID_ANY, wxPoint(0,0), wxSize(ww,hh));
	hh = MYWIN_HGT - 100;
	// display region //
	gridPanel = new wxPanel(this, wxID_ANY, wxPoint(0,100), wxSize(ww, hh));

	// grid pad //
	ww -= 20;	hh -= 20;	// margin
	grid = new wxGrid(gridPanel, ID_MYGRID, wxPoint(10,10), wxSize(ww,hh));

	// create girid with 20lines, 10cols
	grid->CreateGrid(MYG_NROW_INI, MYG_NCOL);

	std::string labels[MYG_NCOL] = {
		"PID", "Value", "Addr", "EndP", "Data",
		"Length", "Data(Hex)", "HS", "SPD", "Cont."
	};
	int colsize[MYG_NCOL] = {
		100, 100, 45, 45, 60,
		60, 200, 70, 70, 70
	};
	for(int i=0; i<MYG_NCOL; i++){
		grid->SetColLabelValue(i, labels[i].c_str());
		grid->SetColSize(i, colsize[i]);
	}

	//  button size //
	int bx = 65, by = 27;
	//  button pos //
	int px = 10, py = 10;
	
    wxButton *btn_RUN =
		new wxButton( btnPanel,
					  ID_BTN_RUN,
					  wxT("RUN"),
					  wxPoint(px,py),  wxSize(bx,by)
					  );
	
	py += (by+10);
	wxButton *btn_STOP =
		new wxButton( btnPanel,
					  ID_BTN_STOP,
					  wxT("STOP"),
					  wxPoint(px,py),  wxSize(bx,by)
					  );

	// Run info box //
	wxStaticBox *box_run = new wxStaticBox(btnPanel, wxID_ANY, "Run Status",
									   wxPoint(px+bx+10,10), wxSize(100,60));

	RunText = new wxStaticText(btnPanel, ID_TXT_RUN, " STOP ",wxPoint(px+bx+20,35));
	RunText->SetForegroundColour(*wxRED);

	// simple texts //
	px = 80;
	py = 85;
	SOFText = new wxStaticText(btnPanel, ID_TXT_RUN, "LAST_SOF = ",wxPoint(px,py));
	TimeText = new wxStaticText(btnPanel, ID_TXT_RUN, "LAST_TIME = ",wxPoint(px+200,py));
	
	// Simple Filters //
	px = 220;
	py = 10;
	wxStaticBox *box_fil = new wxStaticBox(btnPanel, wxID_ANY, "Ignore",
									   wxPoint(px,py), wxSize(270,60));

	px +=10; py +=22;
	chk_SOF =
		new wxCheckBox( btnPanel,
						ID_CHK_SOF,
						wxT("SOF"),
						wxPoint(px,py),  wxSize(bx+15,by)
						);
	px += 85;
	chk_PING =
		new wxCheckBox( btnPanel,
						ID_CHK_PING,
						wxT("PING"),
						wxPoint(px,py),  wxSize(bx+15,by)
						);
	px += 85;
	chk_IN_NAK =
		new wxCheckBox( btnPanel,
						ID_CHK_IN_NAK,
						wxT("IN/NAK"),
						wxPoint(px,py),  wxSize(bx+15,by)
						);
	// other buttons //
	px = 650; py = 25;
	wxButton *btn_SAVE =
		new wxButton( btnPanel,
					  ID_BTN_SAVE,
					  wxT("Save"),
					  wxPoint(px,py),  wxSize(bx,by)
					  );
	px += bx+30;
	wxButton *btn_CLEAR =
		new wxButton( btnPanel,
					  ID_BTN_CLEAR,
					  wxT("Clear"),
					  wxPoint(px,py),  wxSize(bx,by)
					  );
	px += bx+10;
	wxButton *btn_QUIT =
		new wxButton( btnPanel,
					  ID_BTN_QUIT,
					  wxT("Quit"),
					  wxPoint(px,py),  wxSize(bx,by)
					  );
	
	ndata = 0;
	InitSettings();
	
	timer.Start(500); // update every a half second
}

MyFrame::~MyFrame(){
	LEU_Disconnect(LE); // for unintented quit
	LEU_End();
}

/// event handlers ///

void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{
	if( LEU_Disconnect(LE) == LEU_FAIL)
		GetErrStatus();
	Close(true);
}


void MyFrame::OnButton(wxCommandEvent& event)
{ // button event //
	int vv;
	int btncode = event.GetId();
	switch(btncode){
	case ID_BTN_RUN:
		std::cout << "Run" << std::endl;
		if(LEU_Measure(LE) == LEU_FAIL)
			GetErrStatus();
		else{
			chk_PING->Disable();
			chk_SOF->Disable();
			chk_IN_NAK->Disable();
		}
		break;
	case ID_BTN_STOP:
		std::cout << "Stop" << std::endl;
		if(LEU_Stop(LE) == LEU_FAIL)
			GetErrStatus();
		else{
			chk_PING->Enable();
			chk_SOF->Enable();
			chk_IN_NAK->Enable();
		}
		break;
	case ID_BTN_CLEAR:
		grid->ClearGrid();
		vv = grid->GetNumberRows();
		if(vv>20) grid->DeleteRows(20,vv);
		ndata = 0;
		grid->MakeCellVisible(0,0);
		break;
	case ID_BTN_QUIT:
		if( LEU_Disconnect(LE) == LEU_FAIL)
			GetErrStatus();
		Close(true);
		break;
	case ID_BTN_SAVE:
		SaveAsDialog();
		break;
	default:
		std::cout << "Error undefined button code" << std::endl;
		break;
	}
}

void MyFrame::OnCheck(wxCommandEvent& event)
{ // checkbox event //
	
	int chkcode = event.GetId();
	int state = event.GetInt();

	if(state){ // filter ON
		switch(chkcode){
		case ID_CHK_SOF:
			f_filter |= LEU_FILTER_SOF;
			break;
		case ID_CHK_PING:
			f_filter |= LEU_FILTER_PING;
			break;
		case ID_CHK_IN_NAK:
			f_filter |= LEU_FILTER_IN_NAK;
			break;
		default:
			std::cout << "Error undefined chkbox code" << std::endl;
			break;
		}
	}
	else{ // filter OFF
		switch(chkcode){
		case ID_CHK_SOF:
			f_filter &= ~(LEU_FILTER_SOF);
			break;
		case ID_CHK_PING:
			f_filter &= ~(LEU_FILTER_PING);
			break;
		case ID_CHK_IN_NAK:
			f_filter &= ~(LEU_FILTER_IN_NAK);
			break;
		default:
			std::cout << "Error undefined chkbox code" << std::endl;
			break;
		}
	}
	if(LEU_SetFilter(LE, f_filter) == LEU_FAIL){
		std::cout << "Filter Setting failed" << std::endl;
		GetErrStatus();
	}
}


void MyFrame::WriteLine(int lineNo, int disp){

	if(disp & DISP_NTIMES){
		grid->SetCellValue(lineNo,9, USBT.GetDispStr(DISP_NTIMES));
		return;
	}
		 
	if(disp & DISP_PID) grid->SetCellValue(lineNo,0, USBT.GetDispStr(DISP_PID));
	if(disp & DISP_VALUE) grid->SetCellValue(lineNo,1, USBT.GetDispStr(DISP_VALUE));
	if(disp & DISP_ADDR) grid->SetCellValue(lineNo,2, USBT.GetDispStr(DISP_ADDR));
	if(disp & DISP_ENDP) grid->SetCellValue(lineNo,3, USBT.GetDispStr(DISP_ENDP));
	if(disp & DISP_DATA){
		grid->SetCellValue(lineNo,4, USBT.GetDispStr(DISP_DATA_ID));
		grid->SetCellValue(lineNo,5, USBT.GetDispStr(DISP_DATA_LEN));
		grid->SetCellValue(lineNo,6, USBT.GetDispStr(DISP_DATA));
	}
	if(disp & DISP_HAND) grid->SetCellValue(lineNo,7, USBT.GetDispStr(DISP_HAND));
	if(disp & DISP_SPD) grid->SetCellValue(lineNo,8, USBT.GetDispStr(DISP_SPD));
	
}

void MyFrame::OnTimer(wxTimerEvent& WXUNUSED(event))
{ // auto timer event (every a half second)//

	static int reading = 0;

	if(reading) return; // prevent duplicated call
	reading = 1;
	
	LEU_data data;
	int nline = grid->GetNumberRows();
	int scroll = 0;
	char buf[100];
	int transflag = 0;
	int dispflag = 0;
	int state;
	int rcount = 0;

	// check status //
	if(LEU_GetState(LE, &state)==LEU_FAIL){
		std::cout << "GetStatus Error" << std::endl;
		GetErrStatus();
	}
	else if(state == LEU_STATE_MEASURING){
		RunText->SetForegroundColour(*wxGREEN);
		RunText->SetLabel("RUNNING");
	}
	else if(state == LEU_STATE_STOP){
		RunText->SetForegroundColour(*wxRED);
		RunText->SetLabel(" STOP ");
		chk_PING->Enable();
		chk_SOF->Enable();
		chk_IN_NAK->Enable();
	}
	else if(state == LEU_STATE_OVERFLOW){
		RunText->SetForegroundColour(*wxRED);
		RunText->SetLabel("OVERFLOW");
	}
	
	// read data //
	while((LEU_Read(LE, &data) != LEU_FAIL) && (rcount<50000)){
		// prevent too much SOF dislay//
		if(data.packet.PID ==LEU_DATA_PID_SOF){
			LEUD_GetFrame(&data, &last_SOF);
			continue;
		}

		rcount ++; // prevent freezing
		
		transflag = USBT.Append(data); // packet -> transaction
		
		if(transflag!=0){
			while(1){
				dispflag = USBT.NextTrans();
 
				if(!dispflag) break;
				if(!(dispflag & DISP_NOLINE) | !ndata) ndata++;
				while(ndata >= nline){
					grid->AppendRows(); // new line
					nline ++;
					scroll ++;
				}
				WriteLine(ndata-1, dispflag);
			}
		}
	}
	if(scroll>0)
		grid->MakeCellVisible(ndata,0);
	
	sprintf(buf,"LAST_SOF = %04x",last_SOF);
	SOFText->SetLabel(buf);
	sprintf(buf,"LAST_TIME = %.5f sec.", USBT.last_time);
	TimeText->SetLabel(buf);

	reading = 0;
}

void MyFrame::InitSettings(){
	std::cout << "Reset hradware Triggers & Filters ..." << std::endl;
	
	// No filter & Triggers //
	f_filter = 0x0;
	
	if(LEU_SetFilter(LE, f_filter) == LEU_FAIL){
		std::cout << "LE Filter init failed" << std::endl;
		GetErrStatus();
	}

	LEU_FILTER NULL_FILT = {0};
	if(LEU_SetFilter_AddrEndp(LE, NULL_FILT) == LEU_FAIL){
		std::cout << "LE Filter init failed" << std::endl;
		GetErrStatus();
	}
	LEU_TRIGGER NULL_TRIG;
	memset(&NULL_TRIG,0,sizeof(NULL_TRIG));
	if(LEU_SetTrigger(LE, NULL_TRIG) == LEU_FAIL){
		std::cout << "LE Trigger init failed" << std::endl;
		GetErrStatus();
	}
	
	std::cout << "Done." << std::endl;
}

void MyFrame::SaveAsDialog(){
	wxFileDialog
		myFD(this, "Save as CSV file", "", "data000.csv",
			 "CSV files (*.csv)|*.csv", wxFD_SAVE|wxFD_OVERWRITE_PROMPT);

	if(myFD.ShowModal() == wxID_CANCEL) return;

	std::ofstream ofs(myFD.GetPath());

	// header
	for(int j=0;j<MYG_NCOL;j++){
		if(j) ofs << ",";
		ofs << grid->GetColLabelValue(j);
	}
	ofs << std::endl;

	// data
	for(int i=0;i<ndata;i++){
		for(int j=0;j<MYG_NCOL;j++){
			if(j) ofs << ",";
			ofs << grid->GetCellValue(i,j);
		}
		ofs << std::endl;
	}
	
	ofs.close();
}
