#include "MYLEU_Util.h"

MyTrans::MyTrans(){
	// init //
	ClearAll();
}

MyTrans::~MyTrans(){
}

void MyTrans::Clear(USBTrans &t_trans){
	t_trans.time = 0;
	t_trans.speed = 0;
	t_trans.split = 0;
	t_trans.event_id = 0;
	t_trans.token_pid = 0;
	t_trans.addr = -1;
	t_trans.endp = -1;
	t_trans.data_pid = 0;
	t_trans.hand_pid = 0;
	t_trans.data_length = 0;
	t_trans.frame = 0;
	t_trans.busreset_time = 0;
	t_trans.trig_no = -1;
	t_trans.vbus_V = 0;
	t_trans.vbus_I = 0;
	t_trans.data[0] ='\0';
	t_trans.endflag = 0;
}

// test if idential transaction or not
int MyTrans::SameTrans(USBTrans &t1, USBTrans &t2){
	int rt = 
		t1.token_pid == t2.token_pid
		&& t1.event_id == t2.event_id
		&& t1.addr == t2.addr
		&& t1.endp == t2.endp
		&& t1.data_pid == t2.data_pid
		&& t1.hand_pid == t2.hand_pid
		&& t1.data_length == t2.data_length
		&& t1.speed == t2.speed;

	if(!t1.data_pid || (t1.data_length < 1)) return rt;
	else {
		for(int i=0; i< t1.data_length; i++){
			rt = ( rt && (t1.data[i]==t2.data[i]));
		}
	}
	return rt;
	
}

void MyTrans::CopyTrans(USBTrans &t1, USBTrans &t2){
	t1.time = t2.time;
	t1.speed = t2.speed;
	t1.split = t2.split;
	t1.event_id = t2.event_id;
	t1.token_pid = t2.token_pid;
	t1.addr = t2.addr;
	t1.endp = t2.endp;
	t1.data_pid = t2.data_pid;
	t1.hand_pid = t2.hand_pid;
	t1.data_length = t2.data_length;
	t1.frame = t2.frame;
	t1.trig_no = t2.trig_no;
	t1.vbus_V = t2.vbus_V;
	t1.vbus_I = t2.vbus_I;
	
	if(t2.data_length < 1) t1.data[0] = '\0';
	else{
		for(int i=0;i<t2.data_length;i++){
			t1.data[i] = t2.data[i];
		}
	}
}

void MyTrans::ClearAll(){
	Clear(trans);
	Clear(trans_prev);
	Clear(trans_last);
	nSameTrans = 0;
	transflag = 0;
	dispflag = 0;
	last_time = 0;
}

//////////////////////////////////////////////////
// Prepare a new transaction
// Decode previous trnsaction before clearing
// return : 2 = prev data exists
//////////////////////////////////////////////////

int MyTrans::NewTrans(){
	int ret = 0;
	CopyTrans(trans_prev, trans);
	Clear(trans);
	
	if(((trans_prev.token_pid != 0) || (trans_prev.event_id !=0))
	   && trans_prev.endflag == 0){
		ret |= MYUSB_ENDFLAG2; // prev data ready to display
		trans_prev.endflag = 1;
	}
	return ret;
}

int MyTrans::IsToken(int pid){
	switch(pid){
	case LEU_DATA_PID_PING:
	case LEU_DATA_PID_IN:
	case LEU_DATA_PID_OUT:
	case LEU_DATA_PID_SETUP:
		return 1;
	}
	return 0;
}

int MyTrans::IsHandShake(int pid){
	switch(pid){
	case LEU_DATA_PID_ACK:
	case LEU_DATA_PID_NAK:
	case LEU_DATA_PID_NYET:
	case LEU_DATA_PID_PRE_ERR: // ERR
	case LEU_DATA_PID_STALL:
		return 1;
	}
	return 0;
}

int MyTrans::IsNewTrans(USBTrans t_trans, int pid){
	switch(pid){
	case LEU_DATA_PID_SOF:
	case LEU_DATA_PID_SPLIT:
	case LEU_DATA_PID_RSV:
		return 1;
	case LEU_DATA_PID_PING:
	case LEU_DATA_PID_IN:
	case LEU_DATA_PID_OUT:
	case LEU_DATA_PID_SETUP:
		if(t_trans.token_pid || !t_trans.split)
			return 1;
		break;
	}
	return 0;
}

int MyTrans::IsEndTrans(int pid){
	if(IsHandShake(pid)) return 1;
	switch(pid){
	case LEU_DATA_PID_SOF:
	case LEU_DATA_PID_RSV:
	case LEU_DATA_PID_PING:
		return 1;
	}
	return 0;
}

//////////////////////////////////////////////////
// Add a packet to current transaction
// switch to new transaction if needed
// 
//////////////////////////////////////////////////

int MyTrans::Append(LEU_data &ldata){
	
	int t_speed = 0;
	float t_time = 0;

	// store valid timestamp
	if(ldata.timestamp != 0xFFFFFFFFFFFFFFFF)
		last_time = ldata.timestamp*1e-9;
	
	///// check status /////
	switch(ldata.status){
	case LEU_DATA_STATUS_BUSRESET:
	case LEU_DATA_STATUS_SUSPEND:
	case LEU_DATA_STATUS_DISCONNECT:
	case LEU_DATA_STATUS_DEVCHIRP:
	case LEU_DATA_STATUS_HUBCHIRP:
	case LEU_DATA_STATUS_VBUS_LOGGER:
	case LEU_DATA_STATUS_TRIGGER:
	case LEU_DATA_STATUS_TRIGGER_EXT:
	case LEU_DATA_STATUS_UNKNOWN:
		// event detected //
		transflag |= NewTrans(); // flush previous event
		
		if(ldata.timestamp != 0xFFFFFFFFFFFFFFFF)
			trans.time = ldata.timestamp*1e-9;
		trans.event_id = ldata.status;

		switch(trans.event_id){
		case LEU_DATA_STATUS_BUSRESET:
			trans.busreset_time = ldata.busreset.time * 1e-3;
			break;
		case LEU_DATA_STATUS_VBUS_LOGGER:
			trans.vbus_V = ldata.vbus.voltage * 1.0;
			trans.vbus_I = ldata.vbus.current * 1.0;
			break;
		case LEU_DATA_STATUS_TRIGGER:
		case LEU_DATA_STATUS_TRIGGER_EXT:
			trans.trig_no = ldata.trigger.no;
			break;
		}
		
		transflag |= MYUSB_ENDFLAG1;
		trans.endflag = 1;
		return transflag;
		
	case LEU_DATA_STATUS_PACKET_LS:
	case LEU_DATA_STATUS_PACKET_FS:
	case LEU_DATA_STATUS_PACKET_HS:
	case LEU_DATA_STATUS_PACKET_UNKNOWN:
		
		///// packet exists /////
		t_speed = ldata.status;
		t_time = ldata.timestamp * 1.0e-9;

		if( (t_speed != LEU_DATA_STATUS_PACKET_HS)
			&& ( ldata.packet.PID == LEU_DATA_PID_PRE_ERR))
			{
				// PRE inserted
				return MYUSB_PREFLAG; // just notify
			}

		if( IsNewTrans(trans, ldata.packet.PID) ){
			transflag |= NewTrans(); // flush previous event
			
			trans.token_pid = ldata.packet.PID;
			trans.speed = t_speed;
			trans.time = t_time;
		}
		
		if( IsToken(ldata.packet.PID)){
			trans.token_pid = ldata.packet.PID; // overwrite split
			trans.speed = t_speed;
			trans.time = t_time;
			LEUD_GetAddrEndp(&ldata, &(trans.addr), &(trans.endp));
		}

		if( IsHandShake(ldata.packet.PID)){
			trans.hand_pid = ldata.packet.PID;
		}
		
		switch(ldata.packet.PID){
		case LEU_DATA_PID_SOF:
			LEUD_GetFrame(&ldata, &(trans.frame));
			break;
		case LEU_DATA_PID_SPLIT:
			trans.split = 1;
			break;
		case LEU_DATA_PID_DATA0:
		case LEU_DATA_PID_DATA1:
		case LEU_DATA_PID_DATA2:
		case LEU_DATA_PID_MDATA:
			trans.data_pid = ldata.packet.PID;
			trans.data_length = LEUD_GetData(&ldata, trans.data, 4096);
			break;	
		default:
			break;
		}

		if(IsEndTrans(ldata.packet.PID)){
			transflag |= MYUSB_ENDFLAG1;
			trans.endflag = 1;
			return transflag;
		}
		
		break;
	default:
		std::cout << "unknown status : " << int(ldata.status) << std::endl;
		break;
	}
	
	return transflag;
}

int MyTrans::GetStrings(int tID){
	USBTrans * t_trans;
	int rdisp = 0x0;
	
	if(tID == 1) t_trans = &trans;
	else if(tID == 2) t_trans = &trans_prev;
	else return 0;
	
	int evt_mode = 0;
	
	if(t_trans->event_id != 0) evt_mode = 1;
	else if(t_trans->token_pid == 0) return 0;

	int pid;
	
	if(evt_mode){
		pid = t_trans->event_id;
		switch(pid){
		case LEU_DATA_STATUS_BUSRESET:
			name_id = NAME_BUSRESET;
			rdisp |= DISP_PID;
			sprintf( buf_value, "%.2f ms", t_trans->busreset_time);
			rdisp |= DISP_VALUE;
			break;
			
		case LEU_DATA_STATUS_SUSPEND:
			name_id = NAME_SUSPEND;
			rdisp |= DISP_PID;
			break;
			
		case LEU_DATA_STATUS_DISCONNECT:
			name_id = NAME_DISCONNECT;
			rdisp |= DISP_PID;
			break;
			
		case LEU_DATA_STATUS_DEVCHIRP:
			name_id = NAME_DEVCHIRP;
			rdisp |= DISP_PID;
			break;
			
		case LEU_DATA_STATUS_HUBCHIRP:
			name_id = NAME_HUBCHIRP;
			rdisp |= DISP_PID;
			break;
			
		case LEU_DATA_STATUS_VBUS_LOGGER:
			name_id = NAME_VBUS;
			rdisp |= DISP_PID;
			rdisp |= DISP_VALUE;
			sprintf( buf_value, "%.2f V : %.1f mA", t_trans->vbus_V, t_trans->vbus_I);
			break;
			
		case LEU_DATA_STATUS_TRIGGER:
			name_id = NAME_TRIG;
			rdisp |= DISP_PID;
			rdisp |= DISP_VALUE;
			sprintf( buf_value,"No. %d", t_trans->trig_no);
			break;
			
		case LEU_DATA_STATUS_TRIGGER_EXT:
			name_id = NAME_TRIGEX;
			rdisp |= DISP_PID;
			rdisp |= DISP_VALUE;
			sprintf( buf_value,"No. %d", t_trans->trig_no);
			break;
		default:
			return 0;
		}
		nSameTrans = 0;
		return rdisp;
	}
	else{
		pid = t_trans->token_pid;
		// skip SOF, PING //
		if(pid == LEU_DATA_PID_SOF && !(FLAG_DISP_SOF)){
			return 0;
		}
		if(pid == LEU_DATA_PID_PING && !(FLAG_DISP_PING)){
			return 0;
		}

		// duplication check //
		if(nSameTrans && SameTrans(*t_trans, trans_last)){
			nSameTrans ++;
			rdisp = DISP_NTIMES | DISP_NOLINE;
			return rdisp;
		}

		switch(t_trans->speed){
		case LEU_DATA_STATUS_PACKET_LS:
			spd_id = NAME_LSPD;
			rdisp |= DISP_SPD;
			break;
		case LEU_DATA_STATUS_PACKET_FS:
			spd_id = NAME_FSPD;
			rdisp |= DISP_SPD;
			break;
		case LEU_DATA_STATUS_PACKET_HS:
			spd_id = NAME_HSPD;
			rdisp |= DISP_SPD;
			break;
		default:
			spd_id = NAME_NONE;
			break;
		}

		switch(t_trans->hand_pid){
		case LEU_DATA_PID_NAK:
			hand_id = NAME_NAK;
			rdisp |= DISP_HAND;
			break;
		case LEU_DATA_PID_ACK:
			hand_id = NAME_ACK;
			rdisp |= DISP_HAND;
			break;
		case LEU_DATA_PID_NYET:
			hand_id = NAME_NYET;
			rdisp |= DISP_HAND;
			break;
		case LEU_DATA_PID_STALL:
			hand_id = NAME_STALL;
			rdisp |= DISP_HAND;
			break;
		case LEU_DATA_PID_PRE_ERR:
			hand_id = NAME_ERR;
			rdisp |= DISP_HAND;
			break;
		default:
			hand_id = NAME_NONE;
			break;
		}

		if(t_trans->data_length > 0){
			switch(t_trans->data_pid){
			case LEU_DATA_PID_DATA0:
				data_id = NAME_DATA0;
				break;
			case LEU_DATA_PID_DATA1:
				data_id = NAME_DATA1;
				break;
			case LEU_DATA_PID_DATA2:
				data_id = NAME_DATA2;
				break;
			case LEU_DATA_PID_MDATA:
				data_id = NAME_MDATA;
				break;
			default:
				data_id = NAME_NONE;
				break;
			}
			rdisp |= DISP_DATA;
			for(int i=0; i<MYDISP_NCHR && i<t_trans->data_length; i++){
				sprintf(&(buf_data[i*3]),"%02x ",t_trans->data[i]);
			}
			if(t_trans->data_length > MYDISP_NCHR){
				sprintf(&(buf_data[MYDISP_NCHR*3]),"...");
			}
		}
		
		switch(pid){
		case LEU_DATA_PID_RSV:
			name_id = NAME_RSV;
			rdisp |= DISP_PID;
			rdisp |= DISP_ADDR;
			rdisp |= DISP_ENDP;
			break;
			
		case LEU_DATA_PID_PING:
			name_id = NAME_PING;
			rdisp |= DISP_PID;
			rdisp |= DISP_ADDR;
			rdisp |= DISP_ENDP;
			break;
			
		case LEU_DATA_PID_SOF:
			name_id = NAME_SOF;
			rdisp |= DISP_PID;
			rdisp |= DISP_VALUE;
			if(trans_last.token_pid == LEU_DATA_PID_SOF)
				rdisp |= DISP_NOLINE;
			sprintf( buf_value,"%04x", t_trans->frame);
			break;
		case LEU_DATA_PID_SPLIT:
			name_id = NAME_SPLIT;
			rdisp |= DISP_PID;
			break;
		case LEU_DATA_PID_PRE_ERR:
			break;
		case LEU_DATA_PID_IN:
			name_id = NAME_IN;
			rdisp |= DISP_PID;
			rdisp |= DISP_ADDR;
			rdisp |= DISP_ENDP;
			break;
		case LEU_DATA_PID_OUT:
			name_id = NAME_OUT;
			rdisp |= DISP_PID;
			rdisp |= DISP_ADDR;
			rdisp |= DISP_ENDP;
			break;
		case LEU_DATA_PID_SETUP:
			name_id = NAME_SETUP;
			rdisp |= DISP_PID;
			rdisp |= DISP_ADDR;
			rdisp |= DISP_ENDP;
			break;
		default:
			return 0;
		}
		if(rdisp) nSameTrans = 1;
		return rdisp;
	}
	
	return 0;
}

const char * MyTrans::GetDispStr(int did){
	switch(did){
	case DISP_PID:
		return NAME_DISP[name_id];
	case DISP_VALUE:
		return buf_value;
	case DISP_ADDR:
		sprintf(buf_tmp,"%d",trans_last.addr);
		return buf_tmp;
	case DISP_ENDP:
		sprintf(buf_tmp,"%d",trans_last.endp);
		return buf_tmp;
	case DISP_DATA:
		return buf_data;
	case DISP_DATA_ID:
		return NAME_DISP[data_id];
	case DISP_DATA_LEN:
		sprintf(buf_tmp,"%d",trans_last.data_length);
		return buf_tmp;
	case DISP_HAND:
		return NAME_DISP[hand_id];
	case DISP_SPD:
		return NAME_DISP[spd_id];
	case DISP_NTIMES:
		sprintf(buf_tmp,"x %d",nSameTrans);
		return buf_tmp;
	}
	return NULL;
}

int MyTrans::NextTrans(){
	if(!transflag) return 0;
	int rdisp;

	if(transflag & MYUSB_ENDFLAG2){ // previous transaction finished
		transflag &= ~MYUSB_ENDFLAG2; // clear flag
		rdisp = GetStrings(2);
		if(rdisp){
			CopyTrans(trans_last,trans);
			dispflag = rdisp;
			return rdisp;
		}
	}
	
	if(transflag & MYUSB_ENDFLAG1){ // current transaction finished
		transflag &= ~MYUSB_ENDFLAG1; // clear flag
		rdisp = GetStrings(1);
		if(rdisp){
			CopyTrans(trans_last,trans);
			dispflag = rdisp;
			return rdisp;
		}
	}
	return 0;
}


void GetErrStatus(){
	int error;
	
	LEU_GetLastError(&error);
	std::cout << "Error code : " << error << std::endl;
	switch(error){
	case LEU_ERROR_NOINIT:
		std::cout << "have not initialized" << std::endl;;
		break;
	case LEU_ERROR_NOCONNECT:
		std::cout << "not connected" << std::endl;;
		break;
	case LEU_ERROR_NOMEASURE:
		std::cout << "not allowed while measuring" << std::endl;;
		break;
	case LEU_ERROR_MEASURING:
		std::cout << "now measuring" << std::endl;;
		break;
	case LEU_ERROR_PARAMETER:
		std::cout << "parameter error" << std::endl;;
		break;
	case LEU_ERROR_NOSERIAL:
		std::cout << "no device has such a serial #" << std::endl;;
		break;
	case LEU_ERROR_COM:
		std::cout << "connection error" << std::endl;;
		break;
	case LEU_ERROR_ALLOC:
		std::cout << "buffer allocation error" << std::endl;;
		break;
	case LEU_ERROR_TIMEOUT:
		std::cout << "process time-out" << std::endl;;
		break;
	case LEU_ERROR_VERSION:
		std::cout << "too old device version" << std::endl;;
		break;
	case LEU_ERROR_NODATA:
		std::cout << "no data" << std::endl;;
		break;
	case LEU_NOERROR:
	default:
		std::cout << "unknown error" << std::endl;
		break;
	}
};
